#ifndef AUI_H
#define AUI_H
#include "Interface.h"

#ifndef NOCURSES
#include <curses.h>
#else
#define WINDOW void
#define chtype int
#define KEY_MAX DBASE+32
#endif

class shASCIIInterface : public shInterface
{
 public:
    shASCIIInterface ();
    ~shASCIIInterface ();

    int getChar ();
    int getStr (char *buf, int len, const char *prompt,
                const char *dflt = NULL);
    const char *getKeyForCommand (Command cmd);

    void cursorOnXY (int x, int y);
    virtual void drawSideWin ();
    void drawScreen ();
    void drawLog ();
    void refreshScreen ();

    /* Draws everything specified unquestionably. */
    void draw (int x, int y, shSpecialEffect e, shCreature *c,
               shObjectIlk *o, shFeature *f, shSquare *s);
    /* Draws everything specified. What is not given is pulled from
       hero's memory of square contents. */
    void drawMem (int x, int y, shSpecialEffect e, shCreature *c,
                  shObjectIlk *o, shFeature *f, shSquare *s);
    void drawMem (int x, int y) {
        drawMem (x, y, kNone, NULL, NULL, NULL, NULL);
    };

    int diag (const char *format, ...); /* Diagnostic message output to log. */
    void pageLog ();
    void showHistory ();
    void showKills ();
    void showHelp ();
    void showVersion ();
    void editOptions ();
    void keymapChoice ();
    void doScreenShot (int fileHandle);
    void showHighScores ();

    void setColor (shColor c) { mColor = c; };

    shMenu *newMenu (const char *prompt, int flags);
    shTextViewer *newViewer (const char *fname);
    shTextViewer *newViewer (char *lines, int num, int max);


 protected:

    int mX0;
    int mY0;
    int mXMax;
    int mYMax;

    WINDOW *mMainWin;
    WINDOW *mSideWin;
    WINDOW *mLogWin;
    WINDOW *mDiagWin; //diagnostics

    chtype mSqGlyphs[kMaxTerrainType + 1];
    Command mKey2Cmd[KEY_MAX];
    const char *mCommandHelp[kMaxCommand];

    shColor mColor;

    int drawSpecialEffect (shSpecialEffect e);
    chtype featureGlyph (shFeature *f);
    void putGlyph (const shGlyph g);

    //helper functions:
    void initializeGlyphs ();
    void outChar (char ch);
    void outStr (const char *str);
    void moreBegin ();
    void moreEnd ();
    void logPrint (const char *str);
    Command keyToCommand (int key);
    void readKeybindings (const char *fname);
    void assign (int key, Command cmd);
    void resetKeybindings ();
    virtual void pSideWin (int y, int x, const char *fmt, ...);
    virtual void setSideWinCol (shColor color);
    void drawSideWinContent ();
};

struct shMenuChoice;

class shASCIIMenu : public shMenu
{
 public:
    shASCIIMenu (const char *prompt, int flags);
    ~shASCIIMenu ();

    void addPtrItem (char letter, const char *text, const void *value,
                     int count = 1, int selected = 0);
    void addIntItem (char letter, const char *text, const int value,
                     int count = 1, int selected = 0);
    void addHeader (const char *text);
    void addPageBreak ();
    int getPtrResult (const void **value, int *count = NULL);
    int getIntResult (int *value, int *count = NULL);
    void dropResults ();
    void attachHelp (const char *fname);
    void finish ();

 protected:
    char mPrompt[80];
    int mHeight;     /* Viewable rows. */
    int mItemHeight; /* Viewable rows with choices. */
    int mWidth;      /* Viewable cols. */
    int mOffset;     /* First choice. */
    int mLast;       /* Last choice. */
    shVector <shMenuChoice *> mChoices;
    int mNum;        /* Holds number of items to select. */
    int mFlags;
    int mDone;
    int mResultIterator;
    shObjectType mObjTypeHack;
    const char *mHelpFileName;

    virtual void accumulateResults ();
    void select (int i1, int i2, int action, shObjectType t = kUninitialized);
    void showHelp (WINDOW *win);
    shMenuChoice *getResultChoice ();
    const char **prepareHelp (int *lines);
    bool interpretKey (int key, shInterface::Command cmd = shInterface::kNoCommand);
};

class shASCIITextViewer : public shTextViewer
{
 public:
    shASCIITextViewer (const char *fname);
    shASCIITextViewer (char *lines, int num, int max);
    ~shASCIITextViewer ();
    void show ();
 protected:
    char *mLines;
    int mNumLines;
    int mMaxWidth;
 private:
    void print (WINDOW *win, int winline, int fileline);
};
#endif
