#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>

#include "GUI.h"


shGUI::shGUI ()
{
    const int SCREEN_WIDTH = 800;
    const int SCREEN_HEIGHT = 600;
    const int SCREEN_BPP = 32;

    bool error = false;
    screen = NULL;

    if (!SDL_Init (SDL_INIT_VIDEO)) {
        error = true;
    }

    if (!error) {
        screen = SDL_SetVideoMode (SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_SWSURFACE);
        if (!screen) {
            error = true;
            SDL_Quit ();
        }
    }

    if (!error) {
        if (!TTF_Init ()) {
            error = true;
            SDL_FreeSurface (screen);
            SDL_Quit ();
        }
    }

    if (error) {
        fprintf (stderr, "SDL init failed. Try --ascii parameter for ASCII mode.\n");
        exitPRIME (-1);
    }

    mColor = kGray;

    mXMax = 64;
    mYMax = 20;

    mLogSize = 5;
    mLogRow = 0;
    mHistoryIdx = 0;
    mHistoryWrapped = 0;
    mDirty = 0;
    mNoNewline = 0;
    mLogSCount = 0;
    mPause = 0;

    mX0 = 0;
    mY0 = 0;

    const char *name;
    name = NULL;
    name = getenv ("USER");
    if (!name) name = getenv ("USERNAME");
    char dbgfilename[40];
    snprintf (dbgfilename, 40, "%s/dbg.%s.txt", UserDir, name);
    mDbgFile = fopen (dbgfilename, "w");
    if (!mDbgFile) {
        endwin ();
        fprintf (stderr, "Sorry, couldn't open %s\n", dbgfilename);
        exitPRIME (-1);
    }
    setvbuf (mDbgFile, (char *) NULL, _IOLBF, 0);

    initializeCommands ();
    initializeGlyphs ();
}

shGUI::~shGUI ()
{
    TTF_Quit ();
    SDL_Quit ();
}

